import time
from redis.sentinel import Sentinel
import redis


def init_redis(url, **kwargs):
    if not url.lower().startswith(("redis://", "rediss://", "unix://")):
        url = 'redis://' + url

    connection_pool = redis.ConnectionPool.from_url(url, **kwargs)
    print(connection_pool)


# init_redis('127.0.0.1/3:6319')
# init_redis('127.0.0.1/3')
# init_redis('127.0.0.1:6379')
init_redis('localhost:6379', decode_responses=True)


def purchase_item(conn, buyerid, itemid, sellerid, lprice):
    buyer = "users:%s" % buyerid
    seller = "users:%s" % sellerid
    item = "%s.%s" % (itemid, sellerid)
    inventory = "inventory:%s" % buyerid
    end = time.time() + 10
    pipe = conn.pipeline()

    while time.time() < end:    # 条件变化时，重试以检查条件是否满足
        try:
            pipe.watch("market:", buyer)            # 监控条件变化
            price = pipe.zscore("market:", item)
            funds = int(pipe.hget(buyer, "funds"))
            if price != lprice or price > funds:    # 条件不满足，退出
                pipe.unwatch()
                return None

            pipe.multi()        # 开始事务
            pipe.hincrby(seller, "funds", int(price))
            pipe.hincrby(buyer, "funds", int(-price))
            pipe.sadd(inventory, itemid)
            pipe.zrem("market:", item)
            pipe.execute()      # 执行命令
            return True
        except redis.exceptions.WatchError:
            pass

    return False



import time

from src import redisz

# rdz = redisz.Redisz(cluster=True, startup_nodes=[{'host': '10.124.5.222', 'port': 6379},
#                                                  {'host': '10.124.5.190', 'port': 6379},
#                                                  {'host': '10.124.5.191', 'port': 6379}])
rdz = redisz.Redisz('localhost')


def consumer(msg):  # 消息回调函数, msg是收到的消息对象
    data = msg.get('data')
    if type(data) == bytes:
        data = data.decode('utf-8')
    if data == 'exit':
        return False  # 返回False以退订
    print(msg)


rdz.subscribe(['channel1', 'channel2'], consumer, True, {'daemon': True})  # 已线程的方式订阅'channel1', 'channel2'两个频道
print('thread != True 则不会运行到此行代码')

index = 0
while True:
    print('------publish message------')
    rdz.publish('channel1', 'This is a message-' + str(index))  # 向channel1发送消息
    index += 1
    time.sleep(1)



from redis import ResponseError, DataError

from src.unit_test import RedisTestCase


class TestList(RedisTestCase):
    @classmethod
    def get_test_name(cls) -> str:
        return 'list'

    def test_push(self):
        rdz = self.rdz
        self.assertEqual(rdz.list_push('test:numbers', 3, 4), 2)
        self.assertListEqual(rdz.list_getall('test:numbers'), ['3', '4'])
        self.assertEqual(rdz.list_push('test:numbers', [2, 1], left=True), 4)
        self.assertListEqual(rdz.list_getall('test:numbers'), ['1', '2', '3', '4'])
        self.assertEqual(rdz.list_push('test:numbers', [5, 6], 7), 7)
        self.assertListEqual(rdz.list_getall('test:numbers'), ['1', '2', '3', '4', '5', '6', '7'])
        self.assertEqual(rdz.list_push('test:not-exist', 1, xx=True), 0)
        self.assertEqual(rdz.list_getall('test:not-exist'), [])
        self.assertIsNone(rdz.list_getall('test:not-exist', nx_none=True))

    def test_insert(self):
        rdz = self.rdz
        rdz.list_push('test:numbers', [1, 3, 5])

        self.assertEqual(rdz.list_insert('test:numbers', 1, 2), 4)
        self.assertListEqual(rdz.list_getall('test:numbers'), ['1', '2', '3', '5'])
        self.assertEqual(rdz.list_insert('test:numbers', 5, 4, before=True), 5)
        self.assertListEqual(rdz.list_getall('test:numbers'), ['1', '2', '3', '4', '5'])
        self.assertEqual(rdz.list_insert('test:numbers', 10, 11), -1)
        self.assertEqual(rdz.list_insert('test:not-exist', 1, 2), 0)
        self.assertListEqual(rdz.list_getall('test:not-exist'), [])

    def test_set(self):
        rdz = self.rdz
        rdz.list_push('test:numbers', ['1', 'b', 'c'])
        self.assertTrue(rdz.list_set('test:numbers', 1, 2))
        self.assertListEqual(rdz.list_getall('test:numbers'), ['1', '2', 'c'])
        self.assertTrue(rdz.list_set('test:numbers', -1, 3))
        self.assertListEqual(rdz.list_getall('test:numbers'), ['1', '2', '3'])
        with self.assertRaises(ResponseError):
            rdz.list_set('test:not-exist', 1, 2)
        with self.assertRaises(ResponseError):
            rdz.list_set('test:numbers', 10, 2)
        with self.assertRaises(DataError):
            rdz.list_set('test:numbers', 1, None)

    def test_pop(self):
        rdz = self.rdz
        rdz.list_push('test:numbers', ['1', '2', '3', '4', '5', '6'])

        self.assertListEqual(rdz.list_pop('test:numbers', 0), [])

        self.assertEqual(rdz.list_pop('test:numbers'), '6')
        self.assertListEqual(rdz.list_getall('test:numbers'), ['1', '2', '3', '4', '5'])
        self.assertListEqual(rdz.list_pop('test:numbers', 2), ['5', '4'])
        self.assertListEqual(rdz.list_getall('test:numbers'), ['1', '2', '3'])
        self.assertEqual(rdz.list_pop('test:numbers', left=True), '1')
        self.assertListEqual(rdz.list_getall('test:numbers'), ['2', '3'])
        self.assertListEqual(rdz.list_pop('test:numbers', 3), ['3', '2'])
        self.assertListEqual(rdz.list_getall('test:numbers'), [])

        self.assertListEqual(rdz.list_pop('test:not-exist'),[])

    def test_rem(self):
        rdz = self.rdz
        rdz.list_push('test:numbers', ['1', '2', '3', '4', '5', '6', '5', '4', '3', '2', '1'])
        self.assertEqual(rdz.list_rem('test:numbers', 1), 1)
        self.assertListEqual(rdz.list_getall('test:numbers'), ['2', '3', '4', '5', '6', '5', '4', '3', '2', '1'])
        self.assertEqual(rdz.list_rem('test:numbers', 2, -1), 1)
        self.assertListEqual(rdz.list_getall('test:numbers'), ['2', '3', '4', '5', '6', '5', '4', '3', '1'])
        self.assertEqual(rdz.list_rem('test:numbers', 4, 0), 2)
        self.assertListEqual(rdz.list_getall('test:numbers'), ['2', '3', '5', '6', '5', '3', '1'])
        self.assertEqual(rdz.list_rem('test:numbers', 10), 0)
        self.assertListEqual(rdz.list_getall('test:numbers'), ['2', '3', '5', '6', '5', '3', '1'])
        self.assertEqual(rdz.list_rem('test:not-exist', 10), 0)

    def test_index_len(self):
        rdz = self.rdz
        rdz.list_push('test:numbers', ['1', '2', '3', '4', '5', '6'])
        self.assertEqual(rdz.list_index('test:numbers', 1), '2')
        self.assertEqual(rdz.list_index('test:numbers', -1), '6')
        self.assertEqual(rdz.list_index('test:numbers', 10), None)
        self.assertEqual(rdz.list_index('test:not-exist', 0), None)
        self.assertEqual(rdz.list_len('test:numbers'), 6)
        self.assertEqual(rdz.list_len('test:not-exist'), 0)

    def test_range(self):
        rdz = self.rdz
        rdz.list_push('test:numbers', ['1', '2', '3', '4', '5', '6'])
        self.assertListEqual(rdz.list_range('test:numbers', 0, 2), ['1', '2', '3'])
        self.assertListEqual(rdz.list_range('test:numbers', 0, -1), ['1', '2', '3', '4', '5', '6'])
        self.assertListEqual(rdz.list_range('test:numbers', 0, 100), ['1', '2', '3', '4', '5', '6'])
        self.assertListEqual(rdz.list_range('test:not-exist', 0, -1), [])

    def test_iter(self):
        rdz = self.rdz
        numbers = ['1', '2', '3', '4', '5', '6']
        rdz.list_push('test:numbers', numbers)
        index = 0
        for item in rdz.list_iter('test:numbers'):
            with self.subTest():
                self.assertEqual(item, numbers[index])
                index += 1

    def test_bpop(self):
        rdz = self.rdz
        rdz.list_push('test:{numbers}1', ['1', '2'])
        rdz.list_push('test:{numbers}2', ['3', '4'])
        self.assertTupleEqual(rdz.list_bpop(['test:{numbers}1', 'test:{numbers}2']), ('test:{numbers}1', '2'))
        self.assertTupleEqual(rdz.list_bpop(['test:{numbers}1', 'test:{numbers}2']), ('test:{numbers}1', '1'))
        self.assertTupleEqual(rdz.list_bpop(['test:{numbers}1', 'test:{numbers}2']), ('test:{numbers}2', '4'))
        self.assertTupleEqual(rdz.list_bpop(['test:{numbers}1', 'test:{numbers}2']), ('test:{numbers}2', '3'))

        rdz.list_push('test:{numbers}1', ['1', '2'])
        rdz.list_push('test:{numbers}2', ['3', '4'])
        self.assertTupleEqual(rdz.list_bpop(['test:{numbers}1', 'test:{numbers}2'], left=True), ('test:{numbers}1', '1'))
        self.assertTupleEqual(rdz.list_bpop(['test:{numbers}1', 'test:{numbers}2'], left=True), ('test:{numbers}1', '2'))
        self.assertTupleEqual(rdz.list_bpop(['test:{numbers}1', 'test:{numbers}2'], left=True), ('test:{numbers}2', '3'))
        self.assertTupleEqual(rdz.list_bpop(['test:{numbers}1', 'test:{numbers}2'], left=True), ('test:{numbers}2', '4'))

    def test_rpoplpush(self):
        rdz = self.rdz
        rdz.list_push('test:{numbers}1', ['1', '2'])
        rdz.list_push('test:{numbers}2', ['3', '4'])
        self.assertEqual(rdz.list_rpoplpush('test:{numbers}1', 'test:{numbers}2'), '2')
        self.assertListEqual(rdz.list_getall('test:{numbers}1'), ['1'])
        self.assertListEqual(rdz.list_getall('test:{numbers}2'), ['2', '3', '4'])
        self.assertEqual(rdz.list_rpoplpush('test:{numbers}1', 'test:{numbers}2'), '1')
        self.assertListEqual(rdz.list_getall('test:{numbers}1'), [])
        self.assertListEqual(rdz.list_getall('test:{numbers}2'), ['1', '2', '3', '4'])



from src.unit_test import RedisTestCase


class TestSet(RedisTestCase):
    @classmethod
    def get_test_name(cls) -> str:
        return 'set'

    def test_add_rem_pop(self):
        rdz = self.rdz
        self.assertEqual(rdz.set_add('test:letters', 'a', 'b', 'c'), 3)
        self.assertSetEqual(rdz.set_members('test:letters'), {'a', 'b', 'c'})
        self.assertEqual(rdz.set_add('test:letters', ['b', 'c', 'd']), 1)
        self.assertSetEqual(rdz.set_members('test:letters'), {'a', 'b', 'c', 'd'})
        self.assertEqual(rdz.set_add('test:letters', ['c', 'd'], 'e', 'f'), 2)
        self.assertSetEqual(rdz.set_members('test:letters'), {'a', 'b', 'c', 'd', 'e', 'f'})

        self.assertEqual(rdz.set_rem('test:letters', 'a', 'b'), 2)
        self.assertSetEqual(rdz.set_members('test:letters'), {'c', 'd', 'e', 'f'})
        self.assertEqual(rdz.set_rem('test:letters', ['a', 'b', 'c', 'x']), 1)
        self.assertSetEqual(rdz.set_members('test:letters'), {'d', 'e', 'f'})
        self.assertEqual(rdz.set_rem('test:not-exist', ['a', 'b', 'c', 'x']), 0)

        self.assertIsInstance(rdz.set_pop('test:letters'), str)
        self.assertEqual(rdz.set_card('test:letters'), 2)
        self.assertListEqual(rdz.set_pop('test:letters', 0), [])
        self.assertEqual(rdz.set_card('test:letters'), 2)

        self.assertIsInstance(rdz.set_pop('test:letters', 2), list)
        self.assertEqual(rdz.set_card('test:letters'), 0)

        self.assertIsNone(rdz.set_pop('test:letters'))
        self.assertListEqual(rdz.set_pop('test:letters', 2), [])
        self.assertListEqual(rdz.set_pop('test:not-exist', 2), [])

    def test_card_member(self):
        rdz = self.rdz
        rdz.set_add('test:letters', ['a', 'b', 'c', 'd', 'e', 'f'])

        self.assertSetEqual(rdz.set_members('test:letters'), {'a', 'b', 'c', 'd', 'e', 'f'})

        self.assertEqual(rdz.set_card('test:letters'), 6)
        self.assertEqual(rdz.set_card('test:not-exist'), 0)

        self.assertTrue(rdz.set_ismember('test:letters', 'a'))
        self.assertFalse(rdz.set_ismember('test:letters', 'x'))
        self.assertFalse(rdz.set_ismember('test:not-exist', 'a'))

    def test_move(self):
        rdz = self.rdz
        rdz.set_add('test:{letters}1', {'a', 'b', 'c'})
        rdz.set_add('test:{letters}2', {'c', 'd', 'e'})
        self.assertTrue(rdz.set_move('test:{letters}1', 'test:{letters}2', 'a'))
        self.assertSetEqual(rdz.set_members('test:{letters}1'), {'b', 'c'})
        self.assertSetEqual(rdz.set_members('test:{letters}2'), {'a', 'c', 'd', 'e'})
        self.assertTrue(rdz.set_move('test:{letters}1', 'test:{letters}2', 'c'))
        self.assertSetEqual(rdz.set_members('test:{letters}1'), {'b'})
        self.assertSetEqual(rdz.set_members('test:{letters}2'), {'a', 'c', 'd', 'e'})
        self.assertFalse(rdz.set_move('test:{letters}1', 'test:{letters}2', 'f'))

        self.assertFalse(rdz.set_move('test:{not-exist}', 'test:{not-exist}-1', 'f'))

    def test_diff(self):
        rdz = self.rdz
        rdz.set_add('test:{letters}1', {'a', 'b', 'c'})
        rdz.set_add('test:{letters}2', {'b', 'm', 'n'})
        rdz.set_add('test:{letters}3', {'c', 'x', 'y'})

        self.assertSetEqual(rdz.set_diff('test:{letters}1', 'test:{letters}2'), {'c', 'a'})
        self.assertSetEqual(rdz.set_diff(['test:{letters}2', 'test:{letters}3']), {'b', 'm', 'n'})
        self.assertSetEqual(rdz.set_diff(['test:{letters}1', 'test:{letters}2', 'test:{letters}3']), {'a'})
        self.assertSetEqual(rdz.set_diff('test:{letters}1', ['test:{letters}2', 'test:{letters}3']), {'a'})

        self.assertSetEqual(rdz.set_diff('test:{letters}1', 'test:{letters}not-exist'), {'a', 'b', 'c'})
        self.assertSetEqual(rdz.set_diff('test:{letters}not-exist', 'test:{letters}1'), set())

        rdz.list_push('test:diff', [1, 2])
        self.assertEqual(rdz.set_diff(['test:{letters}1', 'test:{letters}2'], dst='test:{letters}diff'), 2)
        self.assertSetEqual(rdz.set_members('test:{letters}diff'), {'a', 'c'})

    def test_inter(self):
        rdz = self.rdz
        rdz.set_add('test:{letters}1', {'a', 'b', 'c'})
        rdz.set_add('test:{letters}2', {'b', 'c', 'd'})
        rdz.set_add('test:{letters}3', {'c', 'd', 'e'})

        self.assertSetEqual(rdz.set_inter(['test:{letters}1', 'test:{letters}2']), {'b', 'c'})
        self.assertSetEqual(rdz.set_inter(['test:{letters}2', 'test:{letters}3']), {'c', 'd'})
        self.assertSetEqual(rdz.set_inter(['test:{letters}1', 'test:{letters}2', 'test:{letters}3']), {'c'})

        self.assertSetEqual(rdz.set_inter('test:{letters}1', 'test:{letters}not-exist'), set())

        rdz.list_push('test:inter', [1, 2])
        self.assertEqual(rdz.set_inter(['test:{letters}1', 'test:{letters}2'], dst='test:{letters}inter'), 2)
        self.assertSetEqual(rdz.set_members('test:{letters}inter'), {'b', 'c'})

    def test_union(self):
        rdz = self.rdz
        rdz.set_add('test:{letters}1', {'a', 'b', 'c'})
        rdz.set_add('test:{letters}2', {'b', 'c', 'd'})
        rdz.set_add('test:{letters}3', {'c', 'd', 'e'})

        self.assertSetEqual(rdz.set_union('test:{letters}1', 'test:{letters}2'), {'a', 'b', 'c', 'd'})
        self.assertSetEqual(rdz.set_union(['test:{letters}2', 'test:{letters}3']), {'b', 'c', 'd', 'e'})
        self.assertSetEqual(rdz.set_union(['test:{letters}1', 'test:{letters}2', 'test:{letters}3']), {'a', 'b', 'c', 'd', 'e'})

        rdz.list_push('test:union', [1, 2])
        self.assertEqual(rdz.set_union(['test:{letters}1', 'test:{letters}2'], dst='test:{letters}union'), 4)
        self.assertSetEqual(rdz.set_members('test:{letters}union'), {'a', 'b', 'c', 'd'})



from redisz import Redisz

rdz = Redisz('localhost')
lock = rdz.lock('my_lock',timeout=1000)
print(lock.acquire())
# with rdz.lock('my_lock'):
#     print('abc')
# print(lock.acquire())

# lock.release()


from redis.exceptions import LockError

from src.unit_test import RedisTestCase


class TestLock(RedisTestCase):
    @classmethod
    def get_test_name(cls) -> str:
        return 'lock'

    # def test_lock(self):
    #     rdz = self.rdz
    #     locked = rdz.acquire_lock('a-lock')
    #     self.assertNotEqual(locked, False)
    #
    #     locked1 = rdz.acquire_lock('a-lock', acquire_seconds=1)
    #     self.assertFalse(locked1)
    #
    #     rdz.release_lock('a-lock', locked)
    #
    #     locked2 = rdz.acquire_lock('a-lock', lock_seconds=1)
    #     self.assertNotEqual(locked2, False)
    #
    #     time.sleep(2)
    #     locked3 = rdz.acquire_lock('a-lock', acquire_seconds=1)
    #     self.assertNotEqual(locked3, False)
    #
    #     rdz.release_lock('a-lock', locked3)

    def test_new_lock(self):
        rdz = self.rdz

        lock = rdz.lock('a-lock', blocking=True)
        lock1 = rdz.lock('a-lock', timeout=1)
        lock2 = rdz.lock('a-lock2', timeout=1)
        self.assertTrue(lock.acquire())
        self.assertFalse(lock1.acquire())
        self.assertTrue(lock2.acquire())
        lock.release()
        with self.assertRaises(LockError):
            lock.release()

        self.assertTrue(lock1.acquire())

        try:
            with lock:
                self.assertTrue(lock.owned())
                pass
        except LockError as err:
            print(err)
        self.assertTrue(lock.acquire())
        lock.release()



from redis.cluster import ClusterNode


def get_list_args(keys, args):
    try:
        iter(keys)
        if isinstance(keys, (bytes, str)):
            keys = [keys]
        else:
            keys = list(keys)
    except TypeError:
        keys = [keys]
    if args and len(args) > 0:
        if isinstance(args[0], (tuple, list)):
            keys.extend(*args)
        else:
            keys.extend(args)
    return keys


def gen_lock_name(lock_name):
    return 'redisz-lock:' + lock_name


def subscribe(rdz, channels, callback):
    pubsub = rdz.get_pubsub()
    pubsub.subscribe(channels)
    for msg in pubsub.listen():
        if callback(msg) is False:
            pubsub.unsubscribe(channels)


def bytes_to_str(value):
    if type(value) is bytes:
        return value.decode('utf-8')
    return value


def create_cluster_nodes(startup_nodes):
    nodes = []
    for item in startup_nodes:
        item_type = type(item)
        if item_type is ClusterNode:
            nodes.append(item)
        else:
            node = {'host': None, 'port': 6379}
            if item_type is str:
                node['host'] = item
            elif item_type is dict:
                node.update(item)
            if node.get('host') is not None:
                nodes.append(ClusterNode(node.get('host'), node.get('port')))
    return nodes


def create_sentinel_nodes(sentinels):
    nodes = []
    for item in sentinels:
        item_type = type(item)
        if item_type is tuple:
            nodes.append(item)
        else:
            node = {'host': None, 'port': 26379}
            if item_type is str:
                node['host'] = item
            elif item_type is dict:
                node.update(item)
            if node.get('host') is not None:
                nodes.append((node.get('host'), node.get('port')))
    return nodes


def parse_url(url: str):
    if type(url) is not str:
        return None
    url = url.strip()
    if url.startswith('sentinel://'):
        sentinels = []
        for item in url[len('sentinel://'):].split(';'):  # 1.1.1.1:26379;2.2.2.2;3.3.3.3:26379
            item = item.strip()
            item_parts = item.split(':')  # 1.1.1.1:26379
            cluster = {'host': item_parts[0]}
            if len(item_parts) > 1:
                cluster['port'] = item_parts[1]
            sentinels.append(cluster)
        return {'mode': 'sentinel', 'nodes': sentinels}
    elif url.startswith('cluster://'):  # 1.1.1.1:7000;2.2.2.2;3.3.3.3:7000
        clusters = []
        for item in url[len('cluster://'):].split(';'):
            item = item.strip()
            item_parts = item.split(':')  # 1.1.1.1:7000
            cluster = {'host': item_parts[0]}
            if len(item_parts) > 1:
                cluster['port'] = item_parts[1]
            clusters.append(cluster)
        return {'mode': 'cluster', 'nodes': clusters}

    if not url.lower().startswith(('redis://', 'rediss://', 'unix://')):
        url = 'redis://' + url

    return {'mode': 'default', 'url': url}



import time

from redis import ResponseError

from src.unit_test import RedisTestCase


class TestStr(RedisTestCase):
    @classmethod
    def get_test_name(cls) -> str:
        return 'global'

    def test_type(self):
        rdz = self.rdz
        rdz.set_value('test:string', 'a')  # str
        rdz.set_value('test:list', [1, 2, 3])  # list
        rdz.set_value('test:hash', {'a': 1, 'b': 2, 'c': 3})  # hash
        rdz.set_value('test:set', {'x', 'y', 'z'})  # set
        rdz.set_value('test:zset', {'x': 1, 'y': 2, 'z': 3}, type='zset')  # zset

        self.assertEqual(rdz.get_type('test:string'), 'string')
        self.assertEqual(rdz.get_type('test:list'), 'list')
        self.assertEqual(rdz.get_type('test:hash'), 'hash')
        self.assertEqual(rdz.get_type('test:set'), 'set')
        self.assertEqual(rdz.get_type('test:zset'), 'zset')
        self.assertEqual(rdz.get_type('test:not-exist'), 'none')

    def test_exists(self):
        rdz = self.rdz
        rdz.set_value('test:string', 'a')  # str
        rdz.set_value('test:list', [1, 2, 3])  # list
        rdz.set_value('test:hash', {'a': 1, 'b': 2, 'c': 3})  # hash
        rdz.set_value('test:set', {'x', 'y', 'z'})  # set
        rdz.set_value('test:zset', {'x': 1, 'y': 2, 'z': 3}, type='zset')  # zset

        self.assertTrue(rdz.exists('test:string'))
        self.assertTrue(rdz.exists('test:string', 'test:list'))
        self.assertTrue(rdz.exists(['test:string', 'test:list']))
        self.assertFalse(rdz.exists('test:string', 'test:not-exist'))

        self.assertEqual(rdz.exists('test:string', 'test:list', return_number=True), 2)
        self.assertEqual(rdz.exists('test:string', 'test:not-exist', return_number=True), 1)

    def test_keys(self):
        rdz = self.rdz
        for i in range(3):
            rdz.set_value('test:x' + str(i), i)
            rdz.set_value('test:y' + str(i), i)

        self.assertSetEqual(set(rdz.keys()), {'test:x0', 'test:x1', 'test:x2', 'test:y0', 'test:y1', 'test:y2'})
        self.assertSetEqual(set(rdz.keys('test:x*')), {'test:x0', 'test:x1', 'test:x2'})
        self.assertSetEqual(set(rdz.keys('*2')), {'test:x2', 'test:y2'})

    def test_delete(self):
        rdz = self.rdz
        for i in range(6):
            rdz.set_value('test:i' + str(i), i)

        self.assertEqual(rdz.delete('test:i0'), 1)
        self.assertEqual(rdz.delete('test:i1', 'test:i2'), 2)
        self.assertEqual(rdz.delete(['test:i3', 'test:i4']), 2)
        self.assertEqual(rdz.delete(['test:i5', 'test:i6']), 1)

    def test_rname(self):
        rdz = self.rdz
        for i in range(6):
            rdz.set_value('test:{i}' + str(i), i)

        self.assertTrue(rdz.rename('test:{i}0', 'test:{i}1'))
        self.assertEqual(rdz.str_get('test:{i}1'), "0")
        self.assertFalse(rdz.rename('test:{i}2', 'test:{i}3', nx=True))
        self.assertEqual(rdz.str_get('test:{i}3'), "3")

    def test_expire(self):
        rdz = self.rdz
        for i in range(6):
            rdz.set_value('test:i' + str(i), i)

        self.assertTrue(rdz.expire('test:i1', 1))
        self.assertFalse(rdz.expire('test:not-exist', 1))

        self.assertEqual(rdz.ttl('test:i1'), 1)
        self.assertEqual(rdz.ttl('test:i2'), -1)
        self.assertEqual(rdz.ttl('test:not-exist'), -2)

        self.assertTrue(rdz.expire('test:i2', 1))
        self.assertTrue(rdz.persist('test:i2'))
        self.assertFalse(rdz.persist('test:not-exist'))
        self.assertEqual(rdz.ttl('test:i2'), -1)

        current_time = int(time.time())
        self.assertTrue(rdz.expireat('test:i3', current_time + 2))  # 确保运行test的客户端跟redis所在server时间一致

        self.assertFalse(rdz.expireat('test:not-exist', current_time + 2))
        self.assertLessEqual(rdz.ttl('test:i3'), 2)
        self.assertGreaterEqual(rdz.ttl('test:i3'), 1)

        self.assertTrue(rdz.exists('test:i3'))
        self.assertTrue(rdz.expireat('test:i3', current_time - 10))
        self.assertFalse(rdz.exists('test:i3'))

        time.sleep(2)
        self.assertFalse(rdz.exists('test:ex1'))

    def test_sort(self):
        rdz = self.rdz
        # if rdz.get_mode() == 'cluster':
        #     print('\n', __name__ + ":test_sort doesn't work with clusters")
        #     return
        rdz.set_value('test:{abc}:sort', [6, 88, 112, 18, 36])
        rdz.set_value('test:{abc}:sort-weight', {'d-6': 1, 'd-88': 2, 'd-112': 3, 'd-18': 4, 'd-36': 5, })
        self.assertListEqual(rdz.sort('test:{abc}:sort'), ['6', '18', '36', '88', '112'])
        self.assertListEqual(rdz.sort('test:{abc}:sort', desc=True), ['112', '88', '36', '18', '6'])
        self.assertListEqual(rdz.sort('test:{abc}:sort', alpha=True), ['112', '18', '36', '6', '88'])
        self.assertListEqual(rdz.sort('test:{abc}:sort', start=1, num=3), ['18', '36', '88'])

        rdz.set_value('test:{abc}:sort-alpha', ['a', 'c', 'b'])
        self.assertListEqual(rdz.sort('test:{abc}:sort-alpha', alpha=True), ['a', 'b', 'c'])
        with self.assertRaises(ResponseError):
            rdz.sort('test:{abc}:sort-alpha')
        #
        self.assertEqual(rdz.sort('test:{abc}:sort', store='test:{abc}:sort-store'), 5)
        self.assertListEqual(rdz.get_value('test:{abc}:sort-store'), ['6', '18', '36', '88', '112'])

        rdz.set_value('test:{abc}:obj-ids', [1, 3, 2])
        rdz.set_value('test:{abc}:obj-1', {'name': '1a', 'weight': 33})
        rdz.set_value('test:{abc}:obj-2', {'name': '2b', 'weight': 22})
        rdz.set_value('test:{abc}:obj-3', {'name': '3c', 'weight': 11})

        if rdz.get_mode() == 'cluster':
            print('\n', __name__ + ":test_sort - BY option of SORT denied in Cluster mode")
            return
        self.assertListEqual(rdz.sort('test:{abc}:obj-ids', by='test:{abc}:obj-*->weight'), ['3', '2', '1'])
        self.assertListEqual(rdz.sort('test:{abc}:obj-ids', by='test:{abc}:obj-*->weight', get='test:{abc}:obj-*->name'), ['3c', '2b', '1a'])
        self.assertListEqual(rdz.sort('test:{abc}:obj-ids', get='test:{abc}:obj-*->name'), ['1a', '2b', '3c'])
        self.assertListEqual(rdz.sort('test:{abc}:obj-ids', by='nosort', get='test:{abc}:obj-*->name'), ['1a', '3c', '2b'])



from redis import ResponseError

from src.unit_test import RedisTestCase


class TestHash(RedisTestCase):
    @classmethod
    def get_test_name(cls) -> str:
        return 'hash'

    def test_set_get(self):
        rdz = self.rdz
        self.assertTrue(rdz.hash_set('test:taozh', 'name', 'Zhang Tao'))
        self.assertDictEqual(rdz.hash_getall('test:taozh'), {'name': 'Zhang Tao'})
        self.assertTrue(rdz.hash_set('test:taozh', mapping={'age': 18, 'email': 'taozh@cisco.com'}))
        self.assertDictEqual(rdz.hash_getall('test:taozh'), {'name': 'Zhang Tao', 'age': '18', 'email': 'taozh@cisco.com'})
        self.assertFalse(rdz.hash_set('test:taozh', 'email', 'zht@cisco.com', nx=True))
        self.assertDictEqual(rdz.hash_getall('test:taozh'), {'name': 'Zhang Tao', 'age': '18', 'email': 'taozh@cisco.com'})
        self.assertTrue(rdz.hash_set('test:taozh', 'company', 'cisco', nx=True))
        self.assertDictEqual(rdz.hash_getall('test:taozh'), {'name': 'Zhang Tao', 'age': '18', 'email': 'taozh@cisco.com', 'company': 'cisco'})
        self.assertEqual(rdz.hash_get('test:taozh', 'name'), 'Zhang Tao')

        self.assertEqual(rdz.hash_get('test:taozh', 'nx'), None)
        self.assertEqual(rdz.hash_get('test:not-exist', 'name'), None)

        self.assertTrue(rdz.hash_mset('test:taozh', {'department': 'cx', 'email': 'zht@cisco.com'}))
        self.assertDictEqual(rdz.hash_getall('test:taozh'), {'name': 'Zhang Tao', 'age': '18', 'email': 'zht@cisco.com', 'company': 'cisco', 'department': 'cx'})

        self.assertListEqual(rdz.hash_mget('test:taozh', 'name'), ['Zhang Tao'])
        self.assertListEqual(rdz.hash_mget('test:taozh', 'name', 'age'), ['Zhang Tao', '18'])
        self.assertListEqual(rdz.hash_mget('test:taozh', ['name', 'age']), ['Zhang Tao', '18'])
        self.assertListEqual(rdz.hash_mget('test:taozh', ['name', 'age', 'not-exist']), ['Zhang Tao', '18', None])

    def test_del(self):
        rdz = self.rdz
        rdz.hash_set('test:kv', mapping={'k1': '1', 'k2': '2', 'k3': '3', 'k4': '4', 'k5': '5', 'k6': '6', 'k7': '7'})
        self.assertEqual(rdz.hash_del('test:kv', 'k1', 'k2'), 2)
        self.assertDictEqual(rdz.hash_getall('test:kv'), {'k3': '3', 'k4': '4', 'k5': '5', 'k6': '6', 'k7': '7'})
        self.assertEqual(rdz.hash_del('test:kv', ['k3', 'k4']), 2)
        self.assertDictEqual(rdz.hash_getall('test:kv'), {'k5': '5', 'k6': '6', 'k7': '7'})
        self.assertEqual(rdz.hash_del('test:kv', ['k5', 'k-nx']), 1)
        self.assertDictEqual(rdz.hash_getall('test:kv'), {'k6': '6', 'k7': '7'})

        self.assertEqual(rdz.hash_del('test:not-exist', 'k1'), 0)
        self.assertEqual(rdz.hash_del('test:kv', 'k-nx'), 0)

    def test_exist_len_keys_values(self):
        rdz = self.rdz
        rdz.hash_set('test:taozh', mapping={'name': 'Zhang Tao', 'age': '18', 'email': 'taozh@cisco.com'})

        self.assertTrue(rdz.hash_exists('test:taozh', 'name'))
        self.assertFalse(rdz.hash_exists('test:taozh', 'city'))
        self.assertFalse(rdz.hash_exists('test:not-exist', 'name'))

        self.assertEqual(rdz.hash_len('test:taozh'), 3)
        self.assertEqual(rdz.hash_len('test:not-exist'), 0)

        self.assertListEqual(rdz.hash_keys('test:taozh'), ['name', 'age', 'email'])
        self.assertListEqual(rdz.hash_keys('test:not-exist'), [])

        self.assertListEqual(rdz.hash_values('test:taozh'), ['Zhang Tao', '18', 'taozh@cisco.com'])
        self.assertListEqual(rdz.hash_keys('test:not-exist'), [])

    def test_incr(self):
        rdz = self.rdz
        rdz.hash_mset('test:taozh', {'name': 'Zhang Tao', 'email': 'taozh@cisco.com', 'score': 99.9, 'count': 18})
        self.assertEqual(rdz.hash_incr('test:taozh', 'count'), 19)
        self.assertEqual(rdz.hash_get('test:taozh', 'count'), '19')
        self.assertEqual(rdz.hash_incr('test:taozh', 'count', 2), 21)
        self.assertEqual(rdz.hash_get('test:taozh', 'count'), '21')
        self.assertEqual(rdz.hash_incr('test:taozh', 'count', -1), 20)
        self.assertEqual(rdz.hash_get('test:taozh', 'count'), '20')
        self.assertEqual(rdz.hash_incr('test:taozh', 'not-exist'), 1)
        self.assertEqual(rdz.hash_get('test:taozh', 'not-exist'), '1')

        with self.assertRaises(ResponseError):
            rdz.hash_incr('test:taozh', 'email')
        with self.assertRaises(ResponseError):
            rdz.hash_incr('test:taozh', 'score')

    def test_decr(self):
        rdz = self.rdz
        rdz.hash_mset('test:taozh', {'name': 'Zhang Tao', 'email': 'taozh@cisco.com', 'score': 99.9, 'count': 10})
        self.assertEqual(rdz.hash_decr('test:taozh', 'count'), 9)
        self.assertEqual(rdz.hash_get('test:taozh', 'count'), '9')
        self.assertEqual(rdz.hash_decr('test:taozh', 'count', 2), 7)
        self.assertEqual(rdz.hash_get('test:taozh', 'count'), '7')
        self.assertEqual(rdz.hash_decr('test:taozh', 'count', -1), 8)
        self.assertEqual(rdz.hash_get('test:taozh', 'count'), '8')
        self.assertEqual(rdz.hash_decr('test:taozh', 'not-exist'), -1)
        self.assertEqual(rdz.hash_get('test:taozh', 'not-exist'), '-1')

        with self.assertRaises(ResponseError):  # Cluster模式下，不能try...except异常，redis.exceptions.ResponseError: hash value is not an integer
            rdz.hash_decr('test:taozh', 'email')

        with self.assertRaises(ResponseError):  # Cluster模式下，不能try...except异常，redis.exceptions.ResponseError: hash value is not an integer
            rdz.hash_decr('test:taozh', 'score')

    def test_incrfloat(self):
        rdz = self.rdz
        rdz.hash_mset('test:taozh', {'name': 'Zhang Tao', 'email': 'taozh@cisco.com', 'score': 88.8})
        self.assertEqual(rdz.hash_incrfloat('test:taozh', 'score'), 89.8)
        self.assertEqual(rdz.hash_get('test:taozh', 'score'), '89.8')
        self.assertEqual(rdz.hash_incrfloat('test:taozh', 'score', 2), 91.8)
        self.assertEqual(rdz.hash_get('test:taozh', 'score'), '91.8')
        self.assertEqual(rdz.hash_incrfloat('test:taozh', 'score', -1), 90.8)
        self.assertEqual(rdz.hash_get('test:taozh', 'score'), '90.8')
        self.assertEqual(rdz.hash_incrfloat('test:taozh', 'not-exist'), 1)
        self.assertEqual(rdz.hash_get('test:taozh', 'not-exist'), '1')

        with self.assertRaises(ResponseError):
            rdz.hash_incrfloat('test:taozh', 'email')

    def test_decrfloat(self):
        rdz = self.rdz
        rdz.hash_mset('test:taozh', {'name': 'Zhang Tao', 'email': 'taozh@cisco.com', 'score': 88.8})
        self.assertEqual(rdz.hash_decrfloat('test:taozh', 'score'), 87.8)
        self.assertEqual(rdz.hash_get('test:taozh', 'score'), '87.8')
        self.assertEqual(rdz.hash_decrfloat('test:taozh', 'score', 2), 85.8)
        self.assertEqual(rdz.hash_get('test:taozh', 'score'), '85.8')
        self.assertEqual(rdz.hash_decrfloat('test:taozh', 'score', -1), 86.8)
        self.assertEqual(rdz.hash_get('test:taozh', 'score'), '86.8')
        self.assertEqual(rdz.hash_decrfloat('test:taozh', 'not-exist'), -1)
        self.assertEqual(rdz.hash_get('test:taozh', 'not-exist'), '-1')

        with self.assertRaises(ResponseError):
            rdz.hash_decrfloat('test:taozh', 'email')



from src.unit_test import RedisTestCase


class TestZSet(RedisTestCase):
    @classmethod
    def get_test_name(cls) -> str:
        return 'zset'

    def test_add_rem(self):
        rdz = self.rdz
        self.assertEqual(rdz.zset_add('test:zset', {'a': 10, 'b': 20}), 2)
        self.assertListEqual(rdz.zset_getall('test:zset'), [('a', 10), ('b', 20.0)])

        self.assertEqual(rdz.zset_add('test:zset', {'b': 30, 'c': 40}), 1)
        self.assertListEqual(rdz.zset_getall('test:zset'), [('a', 10), ('b', 30.0), ('c', 40.0)])

        self.assertEqual(rdz.zset_add('test:zset', {'c': 50, 'd': 60}, nx=True), 1)
        self.assertListEqual(rdz.zset_getall('test:zset'), [('a', 10), ('b', 30.0), ('c', 40.0), ('d', 60.0)])

        self.assertEqual(rdz.zset_add('test:zset', {'d': 70, 'e': 80}, xx=True), 0)
        self.assertListEqual(rdz.zset_getall('test:zset'), [('a', 10), ('b', 30.0), ('c', 40.0), ('d', 70.0)])

        self.assertEqual(rdz.zset_add('test:zset', {'b': 30, 'c': 400, 'd': 700}, ch=True), 2)

        self.assertEqual(rdz.zset_rem('test:zset', 'a'), 1)
        self.assertListEqual(rdz.zset_getall('test:zset'), [('b', 30.0), ('c', 400.0), ('d', 700.0)])

        self.assertEqual(rdz.zset_rem('test:zset', ['b', 'c', 'e']), 2)
        self.assertListEqual(rdz.zset_getall('test:zset'), [('d', 700.0)])

        self.assertEqual(rdz.zset_rem('test:not-exist', 'a'), 0)

    def test_card_count_rank(self):
        rdz = self.rdz
        rdz.set_value('test:zset', {'a': 10, 'b': 20, 'c': 30}, type='zset')

        self.assertEqual(rdz.zset_count('test:zset', 20, 30), 2)
        self.assertEqual(rdz.zset_count('test:zset', 21, 30), 1)
        self.assertEqual(rdz.zset_count('test:not-exist', 0, 10), 0)
        self.assertEqual(rdz.zset_score('test:zset', 'a'), 10)

        self.assertIsNone(rdz.zset_score('test:zset', 'x'))
        self.assertIsNone(rdz.zset_score('test:not-exist', 'x'))

        self.assertEqual(rdz.zset_rank('test:zset', 'a'), 0)
        self.assertEqual(rdz.zset_rank('test:zset', 'b'), 1)
        self.assertIsNone(rdz.zset_rank('test:zset', 'x'))
        self.assertIsNone(rdz.zset_rank('test:not-exist', 'x'))

    def test_incr(self):
        rdz = self.rdz
        rdz.set_value('test:zset', {'a': 10, 'b': 20, 'c': 30}, type='zset')

        self.assertEqual(rdz.zset_incr('test:zset', 1, 'a'), 11)
        self.assertEqual(rdz.zset_incr('test:zset', 2.2, 'b'), 22.2)
        self.assertEqual(rdz.zset_incr('test:zset', -2, 'c'), 28)
        self.assertEqual(rdz.zset_incr('test:zset', 3, 'e'), 3)

        self.assertListEqual(rdz.zset_getall('test:zset'), [('e', 3), ('a', 11), ('b', 22.2), ('c', 28)])

    def test_decr(self):
        rdz = self.rdz
        rdz.set_value('test:zset', {'a': 10, 'b': 20, 'c': 30}, type='zset')

        self.assertEqual(rdz.zset_decr('test:zset', 1, 'a'), 9.0)
        self.assertEqual(rdz.zset_decr('test:zset', 2.2, 'b'), 17.8)
        self.assertEqual(rdz.zset_decr('test:zset', -2, 'c'), 32.0)
        self.assertEqual(rdz.zset_decr('test:zset', 3, 'e'), -3.0)

        self.assertListEqual(rdz.zset_getall('test:zset'), [('e', -3), ('a', 9), ('b', 17.8), ('c', 32)])

    def test_range(self):
        rdz = self.rdz
        rdz.set_value('test:zset', {'a': 10, 'b': 20, 'c': 30}, type='zset')

        self.assertListEqual(rdz.zset_range('test:zset', 0, -1), ['a', 'b', 'c'])
        self.assertListEqual(rdz.zset_range('test:zset', 0, 1), ['a', 'b'])

        self.assertListEqual(rdz.zset_range('test:zset', 0, -1, desc=True), ['c', 'b', 'a'])
        self.assertListEqual(rdz.zset_range('test:zset', 0, 1, desc=True), ['c', 'b'])

        self.assertListEqual(rdz.zset_range('test:zset', 0, -1, withscores=True), [('a', 10), ('b', 20.0), ('c', 30)])

        self.assertListEqual(rdz.zset_range('test:zset', 0, 20, withscores=True, byscore=True), [('a', 10), ('b', 20.0)])
        self.assertListEqual(rdz.zset_range('test:zset', 20, 0, desc=True, withscores=True, byscore=True), [('b', 20.0), ('a', 10)])
        self.assertListEqual(rdz.zset_range('test:zset', 0, 20, desc=True, withscores=True, byscore=True), [])

        self.assertListEqual(rdz.zset_range('test:zset', 0, -1, withscores=True, score_cast_func=lambda x: str(x) + '%'), [('a', '10%'), ('b', '20%'), ('c', '30%')])

        self.assertListEqual(rdz.zset_revrange('test:zset', 0, -1), ['c', 'b', 'a'])
        self.assertListEqual(rdz.zset_revrange('test:zset', 0, 1), ['c', 'b'])
        self.assertListEqual(rdz.zset_revrange('test:zset', 0, -1, withscores=True), [('c', 30), ('b', 20.0), ('a', 10)])
        self.assertListEqual(rdz.zset_revrange('test:zset', 0, -1, withscores=True, score_cast_func=lambda x: str(x) + '%'), [('c', '30%'), ('b', '20%'), ('a', '10%')])

    def test_range_by_score(self):
        rdz = self.rdz
        rdz.set_value('test:zset', {'a': 10, 'b': 20, 'c': 30, 'd': 40, 'e': 50, 'f': 60}, type='zset')

        self.assertListEqual(rdz.zset_rangebyscore('test:zset', 20, 50), ['b', 'c', 'd', 'e'])
        self.assertListEqual(rdz.zset_rangebyscore('test:zset', 20, 50, withscores=True), [('b', 20.0), ('c', 30.0), ('d', 40.0), ('e', 50.0)])
        self.assertListEqual(rdz.zset_rangebyscore('test:zset', 20, 50, 0, 1, withscores=True), [('b', 20.0)])
        self.assertListEqual(rdz.zset_rangebyscore('test:zset', 20, 50, 1, 2, withscores=True), [('c', 30.0), ('d', 40.0)])
        self.assertListEqual(rdz.zset_rangebyscore('test:zset', 20, 50, 1, 10, withscores=True), [('c', 30.0), ('d', 40.0), ('e', 50.0)])

        self.assertListEqual(rdz.zset_revrangebyscore('test:zset', 50, 20), ['e', 'd', 'c', 'b'])
        self.assertListEqual(rdz.zset_revrangebyscore('test:zset', 50, 20, withscores=True), [('e', 50.0), ('d', 40.0), ('c', 30.0), ('b', 20.0)])
        self.assertListEqual(rdz.zset_revrangebyscore('test:zset', 50, 20, 0, 1, withscores=True), [('e', 50.0)])
        self.assertListEqual(rdz.zset_revrangebyscore('test:zset', 50, 20, 1, 2, withscores=True), [('d', 40.0), ('c', 30.0)])
        self.assertListEqual(rdz.zset_revrangebyscore('test:zset', 50, 20, 1, 10, withscores=True), [('d', 40.0), ('c', 30.0), ('b', 20.0)])

    def test_rem_range(self):
        rdz = self.rdz
        rdz.set_value('test:zset', {'a': 10, 'b': 20, 'c': 30, 'd': 40, 'e': 50, 'f': 60}, type='zset')
        self.assertEqual(rdz.zset_remrangebyrank('test:zset', 0, 3), 4)
        self.assertEqual(rdz.zset_remrangebyrank('test:zset', 10, 20), 0)
        self.assertListEqual(rdz.zset_getall('test:zset'), [('e', 50.0), ('f', 60.0)])

        rdz.delete('test:zset')
        rdz.set_value('test:zset', {'a': 10, 'b': 20, 'c': 30, 'd': 40, 'e': 50, 'f': 60}, type='zset')
        self.assertEqual(rdz.zset_remrangebyrank('test:zset', -3, -1), 3)
        self.assertListEqual(rdz.zset_getall('test:zset'), [('a', 10.0), ('b', 20.0), ('c', 30.0)])

        rdz.delete('test:zset')
        rdz.set_value('test:zset', {'a': 10, 'b': 20, 'c': 30, 'd': 40, 'e': 50, 'f': 60}, type='zset')
        self.assertEqual(rdz.zset_remrangebyrank('test:zset', -3, -1), 3)
        self.assertListEqual(rdz.zset_getall('test:zset'), [('a', 10.0), ('b', 20.0), ('c', 30.0)])

        self.assertEqual(rdz.zset_remrangebyrank('test:not-exist', 0, 2), 0)

        rdz.delete('test:zset')
        rdz.set_value('test:zset', {'a': 10, 'b': 20, 'c': 30, 'd': 40, 'e': 50, 'f': 60}, type='zset')
        self.assertEqual(rdz.zset_remrangebyscore('test:zset', 0, 40), 4)
        self.assertEqual(rdz.zset_remrangebyscore('test:zset', 100, 200), 0)
        self.assertListEqual(rdz.zset_getall('test:zset'), [('e', 50.0), ('f', 60.0)])

        rdz.delete('test:zset')
        rdz.set_value('test:zset', {'a': 10, 'b': 20, 'c': 30, 'd': 40, 'e': 50, 'f': 60}, type='zset')
        self.assertEqual(rdz.zset_remrangebyscore('test:zset', 30, 50), 3)
        self.assertListEqual(rdz.zset_getall('test:zset'), [('a', 10.0), ('b', 20.0), ('f', 60.0)])

        self.assertEqual(rdz.zset_remrangebyscore('test:not-exist', 0, 100), 0)



from src.unit_test import RedisTestCase


class TestExt(RedisTestCase):
    @classmethod
    def get_test_name(cls) -> str:
        return 'ext'

    def test_global(self):
        rdz = self.rdz

        self.assertTrue(rdz.set_value('test:str', 'a'))  # str
        self.assertTrue(rdz.set_value('test:str-number', 1.0))  # number
        self.assertEqual(rdz.set_value('test:list', [1, 2, 3]), 3)  # list
        self.assertEqual(rdz.set_value('test:hash', {'a': 1, 'b': 2, 'c': 3}), 3)  # hash
        self.assertEqual(rdz.set_value('test:set', {'x', 'y', 'z'}), 3)  # set
        self.assertEqual(rdz.set_value('test:zset', {'x': 1, 'y': 2, 'z': 3}, type='zset'), 3)  # zset

        self.assertEqual(rdz.get_value('test:str'), 'a')  # str
        self.assertEqual(rdz.get_value('test:str-number'), '1.0')  # number
        self.assertListEqual(rdz.get_value('test:list'), ['1', '2', '3'])  # list
        self.assertDictEqual(rdz.get_value('test:hash'), {'a': '1', 'b': '2', 'c': '3'})  # hash
        self.assertSetEqual(rdz.get_value('test:set'), {'x', 'y', 'z'})  # set
        self.assertListEqual(rdz.get_value('test:zset'), [('x', 1.0), ('y', 2.0), ('z', 3.0)])  # zset

        self.assertSetEqual(set(rdz.get_names()), {'test:str', 'test:str-number', 'test:list', 'test:hash', 'test:set', 'test:zset'})
        self.assertSetEqual(set(rdz.get_names('*set')), {'test:set', 'test:zset'})
        self.assertSetEqual(set(rdz.get_names('test:*set')), {'test:set', 'test:zset'})
        self.assertSetEqual(set(rdz.get_names('test:str*')), {'test:str', 'test:str-number'})

    def test_list(self):
        rdz = self.rdz
        rdz.set_value('test:list', [1, 2, 3])
        self.assertListEqual(rdz.list_getall('test:list'), ['1', '2', '3'])

        self.assertListEqual(rdz.list_getall('test:not-exist'), [])
        self.assertIsNone(rdz.list_getall('test:not-exist', nx_none=True))

        self.assertTrue(rdz.list_exists('test:list', 1))
        self.assertFalse(rdz.list_exists('test:list', 10))
        self.assertFalse(rdz.list_exists('test:not-exist', 1))

    def test_set(self):
        rdz = self.rdz
        rdz.set_value('test:set', {'x', 'y', 'z'})

        self.assertEqual(rdz.set_len('test:set'), 3)
        self.assertEqual(rdz.set_len('test:not-exist'), 0)

        self.assertTrue(rdz.set_exists('test:set', 'x'))
        self.assertFalse(rdz.set_exists('test:set', 'm'))
        self.assertFalse(rdz.set_exists('test:not-exist', 'x'))

        self.assertSetEqual(rdz.set_getall('test:set'), {'x', 'y', 'z'})
        self.assertSetEqual(rdz.set_getall('test:not-exist'), set())

    def test_zset(self):
        rdz = self.rdz
        rdz.set_value('test:zset', {'x': 1, 'y': 2, 'z': 3}, type='zset')
        self.assertEqual(rdz.zset_index('test:zset', 'x'), 0)
        self.assertEqual(rdz.zset_index('test:zset', 'z'), 2)
        self.assertIsNone(rdz.zset_index('test:zset', 'm'))
        self.assertIsNone(rdz.zset_index('test:not-exist', 'm'))

        self.assertEqual(rdz.zset_len('test:zset'), 3)
        self.assertEqual(rdz.zset_len('test:not-exist'), 0)

        self.assertListEqual(rdz.zset_getall('test:zset'), [('x', 1.0), ('y', 2.0), ('z', 3.0)])
        self.assertListEqual(rdz.zset_getall('test:zset', withscores=False), ['x', 'y', 'z'])

        self.assertTrue(rdz.zset_exists('test:zset', 'x'))
        self.assertTrue(rdz.zset_exists('test:zset', 'y'))
        self.assertFalse(rdz.zset_exists('test:zset', 'm'))
        self.assertFalse(rdz.zset_exists('test:not-exist', 'x'))
